#define the names of our sets
set CROPS;                   # all crops
set REQ_CROPS within CROPS;  # crops with minimum requirements, subset of CROPS
set QUOTA_CROPS within CROPS;# crops with quota, subset of CROPS

#declare our parameters
param max_land >0;
param yield {CROPS};
param selling_price{CROPS};
param planting_cost{CROPS};

param penalty{QUOTA_CROPS};
param quota{QUOTA_CROPS};

param req_amount{REQ_CROPS};
param purch_price{REQ_CROPS};

#declare our variables (all are positive)
var farming{CROPS}>=0;
var selling{CROPS}>=0;
var buying{REQ_CROPS}>=0;
var selling_at_penalty{QUOTA_CROPS}>=0;


#maximize income from selling minus expenses for planting, buying and quota penalties
maximize profit:sum{i in CROPS} selling[i]*selling_price[i]
         - sum{i in REQ_CROPS} buying[i]*purch_price[i]
         - sum{i in QUOTA_CROPS} penalty[i]*selling_at_penalty[i]
         - sum{i in CROPS} planting_cost[i]*farming[i];         

# make sure we pay a penalty for all beets sold above the quota
# note that the penalty of 26$ per ton gives a selling price of 10$ per ton
# above the quota.
subject to  Quotas {i in QUOTA_CROPS}: selling[i]
                            <=quota[i]+ selling_at_penalty[i];

# make sure we got enough of the required crops
subject to Required {i in REQ_CROPS} : farming[i]*yield[i] - selling[i]
                        +buying[i]
                       >=req_amount[i];

# Make sure we do not sell more than we have of crops without requirements
subject to Mass_balance {i in CROPS diff REQ_CROPS}: farming[i]*yield[i]
                       >= selling[i];

# make sure we do not plant too much.
subject to Area: sum{i in CROPS} farming[i] <= max_land;
 
